import {
  AbiFunctionSignatureNotFoundError,
  decodeAbiParameters,
  encodeFunctionData,
  erc20Abi,
  formatAbiItem,
  isAddress,
  slice,
  toFunctionSelector
} from "./chunk-C4Z7MTNY.js";
import "./chunk-4VNS5WPM.js";

// src/constants/index.ts
var INTENT_EXECUTION_TYPES = [
  "GASLESS",
  "SELF_PUBLISH"
];
var chainIds = [
  1,
  // ETH Mainnet
  10,
  // Optimism
  130,
  // Unichain
  137,
  // Polygon
  // 2741,// Abstract
  // 5000,// Mantle
  8453,
  // Base
  42161,
  // Arbitrum
  42220,
  // Celo
  57073
  // Ink
];
var stables = ["USDC", "USDbC", "USDCe", "USDT"];
var stableAddresses = {
  1: {
    USDC: "0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
    USDT: "0xdac17f958d2ee523a2206206994597c13d831ec7"
  },
  10: {
    USDC: "0x0b2c639c533813f4aa9d7837caf62653d097ff85",
    USDCe: "0x7F5c764cBc14f9669B88837ca1490cCa17c31607",
    USDT: "0x94b008aA00579c1307B0EF2c499aD98a8ce58e58"
  },
  130: {
    USDC: "0x078D782b760474a361dDA0AF3839290b0EF57AD6",
    USDT: "0x588CE4F028D8e7B53B687865d6A67b3A54C75518"
  },
  137: {
    USDC: "0x3c499c542cEF5E3811e1192ce70d8cC03d5c3359",
    USDCe: "0x2791Bca1f2de4661ED88A30C99A7a9449Aa84174",
    USDT: "0xc2132d05d31c914a87c6611c10748aeb04b58e8f"
  },
  // 2741: {
  //   USDT: "0x0709f39376deee2a2dfc94a58edeb2eb9df012bd",
  //   USDCe: "0x84a71ccd554cc1b02749b35d22f684cc8ec987e1",
  // },
  // 5000: {
  //   USDC: "0x09bc4e0d864854c6afb6eb9a9cdf58ac190d0df9",
  //   USDT: "0x201EBa5CC46D216Ce6DC03F6a759e8E766e956aE",
  // },
  8453: {
    USDC: "0x833589fCD6eDb6E08f4c7C32D4f71b54bdA02913",
    USDbC: "0xd9aAEc86B65D86f6A7B5B1b0c42FFA531710b6CA"
  },
  42161: {
    USDC: "0xaf88d065e77c8cc2239327c5edb3a432268e5831",
    USDCe: "0xff970a61a04b1ca14834a43f5de4533ebddb5cc8",
    USDT: "0xfd086bc7cd5c481dcc9c85ebe478a1c0b69fcbb9"
  },
  42220: {
    USDC: "0xceba9300f2b948710d2653dd7b07f33a8b32118c",
    USDT: "0x48065fbbe25f71c9282ddf5e1cd6d6a887483d5e"
  },
  57073: {
    USDC: "0xF1815bd50389c46847f0Bda824eC8da914045D14",
    USDT: "0x0200C29006150606B650577BBE7B6248F58470c1"
  }
};

// node_modules/viem/_esm/utils/abi/decodeFunctionData.js
function decodeFunctionData(parameters) {
  const { abi, data } = parameters;
  const signature = slice(data, 0, 4);
  const description = abi.find((x) => x.type === "function" && signature === toFunctionSelector(formatAbiItem(x)));
  if (!description)
    throw new AbiFunctionSignatureNotFoundError(signature, {
      docsPath: "/docs/contract/decodeFunctionData"
    });
  return {
    functionName: description.name,
    args: "inputs" in description && description.inputs && description.inputs.length > 0 ? decodeAbiParameters(description.inputs, slice(data, 4)) : void 0
  };
}

// src/utils/index.ts
import { randomBytes } from "crypto";
function getSecondsFromNow(seconds) {
  return new Date(Date.now() + 1e3 * seconds);
}
function dateToTimestamp(date) {
  return BigInt(Math.floor(date.getTime() / 1e3));
}
function isAmountInvalid(amount) {
  return amount < BigInt(0);
}
function sum(items) {
  return items.reduce((acc, amount) => acc + BigInt(amount), BigInt(0));
}
function generateRandomHex(length = 32) {
  return `0x${randomBytes(length).toString("hex")}`;
}

// src/routes/RoutesService.ts
import { EcoProtocolAddresses } from "@eco-foundation/routes-ts";

// src/config/index.ts
var ECO_SDK_CONFIG = {
  openQuotingBaseUrl: "https://quotes.eco.com",
  isPreprod: false
};

// src/routes/RoutesService.ts
var RoutesService = class {
  constructor({ isPreprod } = {}) {
    this.isPreprod = isPreprod || ECO_SDK_CONFIG.isPreprod || false;
  }
  /**
   * Creates a simple intent.
   *
   * @param {CreateSimpleIntentParams} params - The parameters for creating the simple intent.
   * @param {Hex} params.creator - The address of the intent creator.
   * @param {RoutesSupportedChainId} params.originChainID - The chain ID where the intent originates.
   * @param {RoutesSupportedChainId} params.destinationChainID - The chain ID where the intent is fulfilled.
   * @param {Hex} params.receivingToken - The token address to be received on the destination chain.
   * @param {Hex} params.spendingToken - The token address to be spent on the origin chain.
   * @param {bigint} params.spendingTokenLimit - The maximum amount of spending token to use.
   * @param {bigint} params.amount - The amount of receiving token to transfer.
   * @param {string} [params.prover="HyperProver"] - The type of prover to use.
   * @param {Hex} [params.recipient] - The recipient address (defaults to creator).
   * @param {Date} [params.expiryTime] - The expiry time for the intent (defaults to 90 minutes from now).
   * @returns {IntentType} The created intent.
   * @throws {Error} If the creator address is invalid, the origin and destination chain are the same, the amount is invalid, or the expiry time is in the past. Or if there is no prover for the specified configuration.
   */
  createSimpleIntent({
    creator,
    originChainID,
    destinationChainID,
    receivingToken,
    spendingToken,
    spendingTokenLimit,
    amount,
    recipient = creator,
    prover = "HyperProver",
    expiryTime = getSecondsFromNow(90 * 60)
    // 90 minutes from now
  }) {
    if (!isAddress(creator, { strict: false })) {
      throw new Error("Invalid creator address");
    }
    if (!isAddress(recipient, { strict: false })) {
      throw new Error("Invalid recipient address");
    }
    if (originChainID === destinationChainID) {
      throw new Error("originChainID and destinationChainID cannot be the same");
    }
    if (isAmountInvalid(amount)) {
      throw new Error("Invalid amount");
    }
    if (spendingTokenLimit < BigInt(amount)) {
      throw new Error("Insufficient spendingTokenLimit");
    }
    if (expiryTime < getSecondsFromNow(60)) {
      throw new Error("Expiry time must be 60 seconds or more in the future");
    }
    const data = encodeFunctionData({
      abi: erc20Abi,
      functionName: "transfer",
      args: [recipient, amount]
    });
    return {
      route: {
        salt: generateRandomHex(),
        source: BigInt(originChainID),
        destination: BigInt(destinationChainID),
        inbox: EcoProtocolAddresses[this.getEcoChainId(destinationChainID)].Inbox,
        tokens: [
          {
            token: receivingToken,
            amount
          }
        ],
        calls: [
          {
            target: receivingToken,
            data,
            value: BigInt(0)
          }
        ]
      },
      reward: {
        creator,
        prover: this.getProverContract(prover, originChainID),
        deadline: dateToTimestamp(expiryTime),
        nativeValue: BigInt(0),
        tokens: [
          {
            token: spendingToken,
            amount: spendingTokenLimit
          }
        ]
      }
    };
  }
  /**
   * Creates an intent.
   *
   * @param {CreateIntentParams} params - The parameters for creating the intent.
   * @param {Hex} params.creator - The address of the intent creator.
   * @param {RoutesSupportedChainId} params.originChainID - The chain ID where the intent originates.
   * @param {RoutesSupportedChainId} params.destinationChainID - The chain ID where the intent is fulfilled.
   * @param {IntentCall[]} params.calls - The calls to be executed on the destination chain.
   * @param {IntentToken[]} params.callTokens - The tokens required for the calls on the destination chain.
   * @param {IntentToken[]} params.tokens - The tokens to be used for rewards on the origin chain.
   * @param {string|Hex} [params.prover="HyperProver"] - The type of prover or custom prover address to use.
   * @param {Date} [params.expiryTime] - The expiry time for the intent (defaults to 90 minutes from now).
   * @returns {IntentType} The created intent.
   * @throws {Error} If the creator address is invalid, the origin and destination chain are the same, the calls or tokens are invalid, or the expiry time is in the past.
   */
  createIntent({
    creator,
    originChainID,
    destinationChainID,
    calls,
    callTokens,
    tokens,
    prover = "HyperProver",
    expiryTime = getSecondsFromNow(90 * 60)
    // 90 minutes from now
  }) {
    if (!isAddress(creator, { strict: false })) {
      throw new Error("Invalid creator address");
    }
    if (originChainID === destinationChainID) {
      throw new Error("originChainID and destinationChainID cannot be the same");
    }
    if (!calls.length || calls.some((call) => !isAddress(call.target, { strict: false }) || isAmountInvalid(call.value))) {
      throw new Error("Invalid calls");
    }
    if (!callTokens.length || callTokens.some((token) => !isAddress(token.token, { strict: false }) || isAmountInvalid(token.amount))) {
      throw new Error("Invalid callTokens");
    }
    if (!tokens.length || tokens.some((token) => !isAddress(token.token, { strict: false }) || isAmountInvalid(token.amount))) {
      throw new Error("Invalid tokens");
    }
    if (expiryTime < getSecondsFromNow(60)) {
      throw new Error("Expiry time must be 60 seconds or more in the future");
    }
    return {
      route: {
        salt: generateRandomHex(),
        source: BigInt(originChainID),
        destination: BigInt(destinationChainID),
        inbox: EcoProtocolAddresses[this.getEcoChainId(destinationChainID)].Inbox,
        tokens: callTokens,
        calls
      },
      reward: {
        creator,
        prover: this.getProverContract(prover, originChainID),
        deadline: dateToTimestamp(expiryTime),
        nativeValue: BigInt(0),
        tokens
      }
    };
  }
  /**
   * Returns the EcoChainId for a given chainId, appending "-pre" if the environment is pre-production.
   *
   * @param {RoutesSupportedChainId} chainId - The chain ID to be converted to an EcoChainId.
   * @returns {EcoChainIds} The EcoChainId, with "-pre" appended if the environment is pre-production.
   */
  getEcoChainId(chainId) {
    return `${chainId}${this.isPreprod ? "-pre" : ""}`;
  }
  getProverContract(prover, chainID) {
    let proverContract;
    const ecoChainID = this.getEcoChainId(chainID);
    switch (prover) {
      case "HyperProver": {
        proverContract = EcoProtocolAddresses[ecoChainID].HyperProver;
        break;
      }
      case "StorageProver": {
        const defaultProver = EcoProtocolAddresses[ecoChainID].Prover;
        if (!defaultProver) {
          throw new Error("No default prover found for this chain");
        }
        proverContract = defaultProver;
        break;
      }
      default: {
        proverContract = prover;
      }
    }
    return proverContract;
  }
  /**
   * Gets the address of a stable token on a specific chain.
   *
   * @param {RoutesSupportedChainId} chainID - The chain ID where the stable token is deployed.
   * @param {RoutesSupportedStable} stable - The type of stable token.
   * @returns {Hex} The hexadecimal address of the stable token.
   * @throws {Error} If the stable token is not found on the specified chain.
   */
  static getStableAddress(chainID, stable) {
    const stableAddress = stableAddresses[chainID][stable];
    if (!stableAddress) {
      throw new Error(`Stable ${stable} not found on chain ${chainID}`);
    }
    return stableAddress;
  }
  /**
   * Gets the stable token type from its address on a specific chain.
   *
   * @param {RoutesSupportedChainId} chainID - The chain ID where the stable token is deployed.
   * @param {Hex} address - The hexadecimal address of the stable token.
   * @returns {RoutesSupportedStable | undefined} The type of stable token.
   * @throws {Error} If no stable token is found for the given address on the specified chain.
   */
  static getStableFromAddress(chainID, address) {
    for (const stable in stableAddresses[chainID]) {
      if (stableAddresses[chainID][stable]?.toLowerCase() === address.toLowerCase()) {
        return stable;
      }
    }
    throw new Error(`Stable not found for address ${address} on chain ${chainID}`);
  }
};

// src/quotes/OpenQuotingClient.ts
import axios from "axios";
import axiosRetry from "axios-retry";

// src/quotes/types.ts
var OpenQuotingAPI;
((OpenQuotingAPI2) => {
  let Endpoints;
  ((Endpoints2) => {
    Endpoints2["Quotes"] = "/api/v1/quotes";
    Endpoints2["ReverseQuotes"] = "/api/v1/quotes/reverse";
    Endpoints2["InitiateGaslessIntent"] = "/api/v1/quotes/initiateGaslessIntent";
  })(Endpoints = OpenQuotingAPI2.Endpoints || (OpenQuotingAPI2.Endpoints = {}));
})(OpenQuotingAPI || (OpenQuotingAPI = {}));

// src/quotes/OpenQuotingClient.ts
var OpenQuotingClient = class {
  constructor({ dAppID, customBaseUrl }) {
    this.MAX_RETRIES = 5;
    this.dAppID = dAppID;
    this.axiosInstance = axios.create({
      baseURL: customBaseUrl || ECO_SDK_CONFIG.openQuotingBaseUrl
    });
    axiosRetry(this.axiosInstance, { retries: this.MAX_RETRIES, retryDelay: axiosRetry.linearDelay(1e3) });
  }
  /**
   * Requests quotes for a given intent.
   *
   * @param {RequestQuotesForIntentParams} params - The parameters for requesting quotes.
   * @param {IntentType} params.intent - The intent for which quotes are being requested.
   * @param {string[]} params.intentExecutionTypes - The types of intent execution for which quotes are being requested.
   * @returns {Promise<SolverQuote[]>} A promise that resolves to an array of SolverQuote objects containing the quotes.
   * @throws {Error} If intentExecutionTypes is empty or if the request fails after multiple retries.
   *
   * @remarks
   * This method sends a POST request to the `/api/v1/quotes` endpoint with the provided intent information.
   * The intentData returned in each quote will have the fee added to the reward tokens.
   */
  async requestQuotesForIntent({ intent, intentExecutionTypes = ["SELF_PUBLISH", "GASLESS"] }) {
    if (intentExecutionTypes.length === 0) {
      throw new Error("intentExecutionTypes must not be empty");
    }
    const payload = {
      dAppID: this.dAppID,
      intentExecutionTypes,
      intentData: this.formatIntentData(intent)
    };
    payload.intentData.routeData.salt = "0x0";
    const response = await this.axiosInstance.post(OpenQuotingAPI.Endpoints.Quotes, payload);
    return this.parseQuotesResponse(intent, response.data);
  }
  /**
   * Requests reverse quotes for a given intent.
   * 
   * @param {RequestQuotesForIntentParams} params - The parameters for requesting reverse quotes.
   * @param {IntentType} params.intent - The intent for which quotes are being requested.
   * @param {string[]} params.intentExecutionTypes - The types of intent execution for which quotes are being requested.
   * @returns {Promise<SolverQuote[]>} A promise that resolves to an array of SolverQuote objects containing the quotes.
   * @throws {Error} If intentExecutionTypes is empty, if the calls aren't ERC20.transfer calls, or if the request fails after multiple retries.
   * 
   * @remarks
   * This method sends a POST request to the `/api/v1/quotes/reverse` endpoint with the provided intent information.
   * This intentData returned in each quote will have the fee subtracted from the route tokens and calls.
   */
  async requestReverseQuotesForIntent({ intent, intentExecutionTypes = ["SELF_PUBLISH", "GASLESS"] }) {
    if (intentExecutionTypes.length === 0) {
      throw new Error("intentExecutionTypes must not be empty");
    }
    if (intent.route.calls.some((call) => {
      try {
        const result = decodeFunctionData({ data: call.data, abi: erc20Abi });
        return result.functionName !== "transfer";
      } catch {
        return true;
      }
    })) {
      throw new Error("Reverse quote calls must be ERC20 transfer calls");
    }
    const payload = {
      dAppID: this.dAppID,
      intentExecutionTypes,
      intentData: this.formatIntentData(intent)
    };
    payload.intentData.routeData.salt = "0x0";
    const response = await this.axiosInstance.post(OpenQuotingAPI.Endpoints.ReverseQuotes, payload);
    return this.parseQuotesResponse(intent, response.data);
  }
  formatIntentData(intent) {
    return {
      routeData: {
        salt: intent.route.salt,
        originChainID: intent.route.source.toString(),
        destinationChainID: intent.route.destination.toString(),
        inboxContract: intent.route.inbox,
        tokens: intent.route.tokens.map((token) => ({
          token: token.token,
          amount: token.amount.toString()
        })),
        calls: intent.route.calls.map((call) => ({
          target: call.target,
          data: call.data,
          value: call.value.toString()
        }))
      },
      rewardData: {
        creator: intent.reward.creator,
        proverContract: intent.reward.prover,
        deadline: intent.reward.deadline.toString(),
        nativeValue: intent.reward.nativeValue.toString(),
        tokens: intent.reward.tokens.map((token) => ({
          token: token.token,
          amount: token.amount.toString()
        }))
      }
    };
  }
  parseQuotesResponse(intent, response) {
    return response.data.map((quote) => ({
      quoteID: quote.quoteID,
      solverID: quote.solverID,
      quoteData: {
        quoteEntries: quote.quoteData.quoteEntries.map((entry) => ({
          intentExecutionType: entry.intentExecutionType,
          intentData: {
            route: {
              salt: intent.route.salt,
              source: intent.route.source,
              destination: intent.route.destination,
              inbox: intent.route.inbox,
              tokens: entry.routeTokens.map((token) => ({
                token: token.token,
                amount: BigInt(token.amount)
              })),
              calls: entry.routeCalls.map((call) => ({
                target: call.target,
                data: call.data,
                value: BigInt(call.value)
              }))
            },
            reward: {
              creator: intent.reward.creator,
              prover: intent.reward.prover,
              deadline: intent.reward.deadline,
              nativeValue: intent.reward.nativeValue,
              tokens: entry.rewardTokens.map((token) => ({
                token: token.token,
                amount: BigInt(token.amount)
              }))
            }
          },
          expiryTime: BigInt(entry.expiryTime)
        }))
      }
    }));
  }
  /**
   * Initiates a gasless intent via the Open Quoting service.
   *  
   * @param {InitiateGaslessIntentParams} params - The parameters for initiating a gasless intent.
   * @param {string} params.funder - The address of the entity funding the intent execution.
   * @param {IntentType} params.intent - The intent to be executed gaslessly.
   * @param {string} params.quoteID - The ID of the quote selected for execution.
   * @param {string} params.solverID - The ID of the solver that is executing the intent.
   * @param {string} params.vaultAddress - The address of the vault to use for the gasless intent.
   * @param {PermitData} [params.permitData] - Optional permit data for token approvals.
   * @returns {Promise<InitiateGaslessIntentResponse>} A promise that resolves to the response from the Open Quoting service.
   * @throws {Error} If the request fails (no retries are attempted for this endpoint).
   * 
   * @remarks
   * This method sends a POST request to the `/api/v1/quotes/initiateGaslessIntent` endpoint with the provided intent information.
   */
  async initiateGaslessIntent({ funder, intent, quoteID, solverID, vaultAddress, permitData }) {
    const payload = {
      dAppID: this.dAppID,
      quoteID,
      solverID,
      intentData: {
        ...this.formatIntentData(intent),
        gaslessIntentData: {
          funder,
          vaultAddress,
          permitData: permitData ? this.formatPermitData(permitData) : void 0
        }
      }
    };
    const response = await this.axiosInstance.post(OpenQuotingAPI.Endpoints.InitiateGaslessIntent, payload, {
      "axios-retry": {
        retries: 0
      }
    });
    return response.data.data;
  }
  formatPermitData(permit) {
    if (permit.permit) {
      const permit1 = permit;
      return {
        permit: permit1.permit.map((p) => ({
          token: p.token,
          data: {
            signature: p.data.signature,
            deadline: p.data.deadline.toString()
          }
        }))
      };
    } else {
      const permit2 = permit;
      if (permit2.permit2.permitData.singlePermitData) {
        const permitData = permit2.permit2.permitData;
        return {
          permit2: {
            permitContract: permit2.permit2.permitContract,
            permitData: {
              singlePermitData: {
                typedData: {
                  details: {
                    token: permitData.singlePermitData.typedData.details.token,
                    amount: permitData.singlePermitData.typedData.details.amount.toString(),
                    expiration: permitData.singlePermitData.typedData.details.expiration.toString(),
                    nonce: permitData.singlePermitData.typedData.details.nonce.toString()
                  },
                  spender: permitData.singlePermitData.typedData.spender,
                  sigDeadline: permitData.singlePermitData.typedData.sigDeadline.toString()
                }
              }
            },
            signature: permit2.permit2.signature
          }
        };
      } else {
        const permitData = permit2.permit2.permitData;
        return {
          permit2: {
            permitContract: permit2.permit2.permitContract,
            permitData: {
              batchPermitData: {
                typedData: {
                  details: permitData.batchPermitData.typedData.details.map((detail) => ({
                    token: detail.token,
                    amount: detail.amount.toString(),
                    expiration: detail.expiration.toString(),
                    nonce: detail.nonce.toString()
                  })),
                  spender: permitData.batchPermitData.typedData.spender,
                  sigDeadline: permitData.batchPermitData.typedData.sigDeadline.toString()
                }
              }
            },
            signature: permit2.permit2.signature
          }
        };
      }
    }
  }
};

// src/quotes/quoteSelectors/index.ts
function selectCheapestQuote(solverQuotes, isReverse = false, allowedIntentExecutionTypes = ["SELF_PUBLISH"]) {
  return solverQuotes.reduce(({ solverID: cheapestSolverID, quoteID: cheapestQuoteID, quoteData: cheapestQuoteData }, solverQuoteResponse) => {
    const quotes = solverQuoteResponse.quoteData.quoteEntries;
    let localCheapestQuoteData = cheapestQuoteData;
    let localCheapestSolverID = cheapestSolverID;
    let localCheapestQuoteID = cheapestQuoteID;
    const defaultSum = BigInt(isReverse ? 0 : Number.MAX_SAFE_INTEGER);
    for (const quoteData of quotes) {
      if (allowedIntentExecutionTypes.includes(quoteData.intentExecutionType)) {
        let localCheapestTokens = localCheapestQuoteData?.intentData?.reward.tokens;
        let quoteTokens = quoteData.intentData.reward.tokens;
        if (isReverse) {
          localCheapestTokens = localCheapestQuoteData?.intentData?.route.tokens;
          quoteTokens = quoteData.intentData.route.tokens;
        }
        const localCheapestSum = localCheapestQuoteData ? sum(localCheapestTokens.map(({ amount }) => amount)) : defaultSum;
        const quoteSum = sum(quoteTokens.map(({ amount }) => amount));
        if (isReverse) {
          if (quoteSum > localCheapestSum) {
            localCheapestSolverID = solverQuoteResponse.solverID;
            localCheapestQuoteID = solverQuoteResponse.quoteID;
            localCheapestQuoteData = quoteData;
          }
        } else {
          if (quoteSum < localCheapestSum) {
            localCheapestSolverID = solverQuoteResponse.solverID;
            localCheapestQuoteID = solverQuoteResponse.quoteID;
            localCheapestQuoteData = quoteData;
          }
        }
      }
    }
    if (!cheapestQuoteData) {
      return {
        solverID: localCheapestSolverID,
        quoteID: localCheapestQuoteID,
        quoteData: localCheapestQuoteData
      };
    }
    if (isReverse) {
      const globalTokens = cheapestQuoteData.intentData.route.tokens;
      const localTokens = localCheapestQuoteData?.intentData?.route.tokens;
      const globalSum = sum(globalTokens.map(({ amount }) => amount));
      const localSum = localCheapestQuoteData ? sum(localTokens.map(({ amount }) => amount)) : defaultSum;
      return localSum > globalSum ? {
        solverID: localCheapestSolverID,
        quoteID: localCheapestQuoteID,
        quoteData: localCheapestQuoteData
      } : {
        solverID: cheapestSolverID,
        quoteID: cheapestQuoteID,
        quoteData: cheapestQuoteData
      };
    } else {
      const globalTokens = cheapestQuoteData.intentData.reward.tokens;
      const localTokens = localCheapestQuoteData?.intentData?.reward.tokens;
      const globalSum = sum(globalTokens.map(({ amount }) => amount));
      const localSum = localCheapestQuoteData ? sum(localTokens.map(({ amount }) => amount)) : defaultSum;
      return localSum < globalSum ? {
        solverID: localCheapestSolverID,
        quoteID: localCheapestQuoteID,
        quoteData: localCheapestQuoteData
      } : {
        solverID: cheapestSolverID,
        quoteID: cheapestQuoteID,
        quoteData: cheapestQuoteData
      };
    }
  }, {});
}
export {
  INTENT_EXECUTION_TYPES,
  OpenQuotingClient,
  RoutesService,
  chainIds,
  selectCheapestQuote,
  stableAddresses,
  stables
};
